/* $Id: interface.c 5 2003-08-11 16:19:06Z lennart $ */

/*
 * This file is part of waproamd.
 *
 * waproamd is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * waproamd is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with waproamd; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 */

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include <libdaemon/dlog.h>

#include "interface.h"
#include "util.h"
#include "wireless.h"

struct interface *interface_open(char *name) {
    struct interface* i = NULL;

    if (!(i = (struct interface*) malloc(sizeof(struct interface))))
        return NULL;

    memset(i, 0, sizeof(struct interface));
    strncpy(i->name, name, IFNAMSIZ);

    if ((i->fd = socket(PF_INET, SOCK_DGRAM, 0)) < 0) {
        daemon_log(LOG_ERR, "socket(): %s\n", strerror(errno));
        goto fail;
    }
        
    return i;
    
fail:

    if (i)
        interface_close(i);

    return NULL;
}

void interface_close(struct interface *i) {
    assert(i);

    if (i->fd >= 0)
        close(i->fd);

    free(i);
}

int interface_is_assoc(struct interface *i, struct hw_addr *ap) {
    struct hw_addr ap2;
    struct iwreq req;
    struct iw_statistics q;

    assert(i);

    if (ap)
        memset(ap->addr, 0, ETH_ALEN);

    memset(&req, 0, sizeof(req));
    strncpy(req.ifr_ifrn.ifrn_name, i->name, IFNAMSIZ);
     
    if (ioctl(i->fd, SIOCGIWAP, &req) < 0) {
        daemon_log(LOG_ERR, "Failed to get AP address\n");
        return -1;
    }

    memcpy(ap2.addr, &(req.u.ap_addr.sa_data), ETH_ALEN);
 
    if (!is_assoc_ap(&ap2))
        return 0;

    memset(&req, 0, sizeof(req));
    strncpy(req.ifr_ifrn.ifrn_name, i->name, IFNAMSIZ);

    req.u.data.pointer = (caddr_t) &q;
    req.u.data.length = sizeof(q);
    req.u.data.flags = 1;

    if (ioctl(i->fd, SIOCGIWSTATS, &req) < 0) {
        daemon_log(LOG_ERR, "Failed to get interface quality\n");
        return -1;
    }

    if (q.qual.qual <= 0)
        return 0;

    if (ap)
        memcpy(ap->addr, ap2.addr, ETH_ALEN);
    
    return 1;
}
